#!/usr/bin/python3

from bcc import BPF
from time import sleep

# This is C code!
BPF_PROGRAM = """
BPF_HASH(uids);

int handler_function(void *ctx) {
    u64 times = 0;
    u64 *pointer;

    u64 uid;
    uid = bpf_get_current_uid_gid() & 0xFFFFFFFF;
    // u32 pid = bpf_get_current_pid_tgid();

    pointer = uids.lookup(&uid);
    if (pointer !=0)
        times = *pointer;

    times++;
    uids.update(&uid, &times);

    // If you want to print the output as it comes
    // bpf_trace_printk("uid: %d, pid: %d %d\\n", uid, pid);
    return 0;
}
"""

b = BPF(text=BPF_PROGRAM)
my_event = b.get_syscall_fnname("getuid")
b.attach_kprobe(event=my_event, fn_name="handler_function")

# In case you want to print the generated output
# b.trace_print()

while True:
    values = b["uids"].items()
    if len(values) == 0:
        print
    else:
        for k, v in values:
            print("uid:", k.value, "times:", v.value)
        print()

    sleep(10)
